@@ -26,6 +26,8 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "../../src/client/event_queue.h"
 #include "../../src/client/registry.h"
 #include "../../src/client/output.h"
+#include "../../src/client/pointerconstraints.h"
+#include "../../src/client/pointergestures.h"
 #include "../../src/client/seat.h"
 #include "../../src/client/relativepointer.h"
 #include "../../src/client/server_decoration.h"
@@ -46,6 +48,8 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "../../src/server/subcompositor_interface.h"
 #include "../../src/server/outputmanagement_interface.h"
 #include "../../src/server/outputdevice_interface.h"
+#include "../../src/server/pointerconstraints_interface.h"
+#include "../../src/server/pointergestures_interface.h"
 #include "../../src/server/textinput_interface.h"
 #include "../../src/server/xdgshell_interface.h"
 #include "../../src/server/relativepointer_interface.h"
@@ -57,6 +61,8 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include <wayland-text-input-v2-client-protocol.h>
 #include <wayland-xdg-shell-v5-client-protocol.h>
 #include <wayland-relativepointer-unstable-v1-client-protocol.h>
+#include <wayland-pointer-gestures-unstable-v1-client-protocol.h>
+#include <wayland-pointer-constraints-unstable-v1-client-protocol.h>
 
 class TestWaylandRegistry : public QObject
 {
@@ -84,6 +90,8 @@ private Q_SLOTS:
     void testBindTextInputManagerUnstableV2();
     void testBindXdgShellUnstableV5();
     void testBindRelativePointerManagerUnstableV1();
+    void testBindPointerGesturesUnstableV1();
+    void testBindPointerConstraintsUnstableV1();
     void testGlobalSync();
     void testGlobalSyncThreaded();
     void testRemoval();
@@ -106,6 +114,8 @@ private:
     KWayland::Server::TextInputManagerInterface *m_textInputManagerV2;
     KWayland::Server::XdgShellInterface *m_xdgShellUnstableV5;
     KWayland::Server::RelativePointerManagerInterface *m_relativePointerV1;
+    KWayland::Server::PointerGesturesInterface *m_pointerGesturesV1;
+    KWayland::Server::PointerConstraintsInterface *m_pointerConstraintsV1;
 };
 
 static const QString s_socketName = QStringLiteral("kwin-test-wayland-registry-0");
@@ -126,6 +136,8 @@ TestWaylandRegistry::TestWaylandRegistry(QObject *parent)
     , m_textInputManagerV2(nullptr)
     , m_xdgShellUnstableV5(nullptr)
     , m_relativePointerV1(nullptr)
+    , m_pointerGesturesV1(nullptr)
+    , m_pointerConstraintsV1(nullptr)
 {
 }
 
@@ -170,6 +182,12 @@ void TestWaylandRegistry::init()
     m_relativePointerV1 = m_display->createRelativePointerManager(KWayland::Server::RelativePointerInterfaceVersion::UnstableV1);
     m_relativePointerV1->create();
     QCOMPARE(m_relativePointerV1->interfaceVersion(), KWayland::Server::RelativePointerInterfaceVersion::UnstableV1);
+    m_pointerGesturesV1 = m_display->createPointerGestures(KWayland::Server::PointerGesturesInterfaceVersion::UnstableV1);
+    m_pointerGesturesV1->create();
+    QCOMPARE(m_pointerGesturesV1->interfaceVersion(), KWayland::Server::PointerGesturesInterfaceVersion::UnstableV1);
+    m_pointerConstraintsV1 = m_display->createPointerConstraints(KWayland::Server::PointerConstraintsInterfaceVersion::UnstableV1);
+    m_pointerConstraintsV1->create();
+    QCOMPARE(m_pointerConstraintsV1->interfaceVersion(), KWayland::Server::PointerConstraintsInterfaceVersion::UnstableV1);
 }
 
 void TestWaylandRegistry::cleanup()
@@ -317,6 +335,16 @@ void TestWaylandRegistry::testBindRelativePointerManagerUnstableV1()
     TEST_BIND(KWayland::Client::Registry::Interface::RelativePointerManagerUnstableV1, SIGNAL(relativePointerManagerUnstableV1Announced(quint32,quint32)), bindRelativePointerManagerUnstableV1, zwp_relative_pointer_manager_v1_destroy)
 }
 
+void TestWaylandRegistry::testBindPointerGesturesUnstableV1()
+{
+    TEST_BIND(KWayland::Client::Registry::Interface::PointerGesturesUnstableV1, SIGNAL(pointerGesturesUnstableV1Announced(quint32,quint32)), bindPointerGesturesUnstableV1, zwp_pointer_gestures_v1_destroy)
+}
+
+void TestWaylandRegistry::testBindPointerConstraintsUnstableV1()
+{
+    TEST_BIND(KWayland::Client::Registry::Interface::PointerConstraintsUnstableV1, SIGNAL(pointerConstraintsUnstableV1Announced(quint32,quint32)), bindPointerConstraintsUnstableV1, zwp_pointer_constraints_v1_destroy)
+}
+
 #undef TEST_BIND
 
 void TestWaylandRegistry::testRemoval()
@@ -509,24 +537,37 @@ void TestWaylandRegistry::testDestroy()
     QVERIFY(connectedSpy.wait());
 
     Registry registry;
+    QSignalSpy shellAnnouncedSpy(&registry, SIGNAL(shellAnnounced(quint32,quint32)));
+
     QVERIFY(!registry.isValid());
-    registry.create(connection.display());
+    registry.create(&connection);
     registry.setup();
     QVERIFY(registry.isValid());
 
-    connect(&connection, &ConnectionThread::connectionDied, &registry, &Registry::destroy);
+    //create some arbitrary Interface
+    shellAnnouncedSpy.wait();
+    QScopedPointer<Shell> shell(registry.createShell(registry.interface(Registry::Interface::Shell).name, registry.interface(Registry::Interface::Shell).version, &registry));
+
 
     QSignalSpy connectionDiedSpy(&connection, SIGNAL(connectionDied()));
+    QSignalSpy registryDiedSpy(&registry, SIGNAL(registryDestroyed()));
+
     QVERIFY(connectionDiedSpy.isValid());
+    QVERIFY(registryDiedSpy.isValid());
+
     delete m_display;
     m_display = nullptr;
     QVERIFY(connectionDiedSpy.wait());
 
+    QVERIFY(connectionDiedSpy.count() == 1);
+    QVERIFY(registryDiedSpy.count() == 1);
+
     // now the registry should be destroyed;
     QVERIFY(!registry.isValid());
 
     // calling destroy again should not fail
     registry.destroy();
+    shell->destroy();
 }
 
 void TestWaylandRegistry::testGlobalSync()
